/**
 * Diese Klasse stellt Lager zur Verfgung. Lager enthalten verschiedene Lagerpltze und knnen Waren lagern.
 * 
 * @author SK
 * 
 */
public class Lager {
	private Lagerplatz[] lager_25; // Lagerpltze mit Raumtemperatur.
	private Lagerplatz[] lager_5; // gekhlte Lagerpltze.
	private Lagerplatz[] lager_minus18; // auf -18C tiefgekhlte Lagerpltze.
	private Lagerplatz[] lager_minus30; // auf -30C tiefgekhlte Lagerpltze.

	/**
	 * Dieser Konstruktor erzeugt ein neues Lager mit einer bestimmten Anzahl an verschiedenen Lagerpltzen.
	 * Vorbedingungen: a, b, c, d > 0
	 * 
	 * @param a
	 *            Anzahl der Lagerpltze mit Raumtemperatur.
	 * @param b
	 *            Anzahl der gekhlten Lagerpltze.
	 * @param c
	 *            Anzahl der auf -18C tiefgekhlten Lagerpltze.
	 * @param d
	 *            Anzahl der auf -30C tiefgekhlten Lagerpltze.
	 */
	public Lager(int a, int b, int c, int d) {
		this.lager_25 = new Lagerplatz_Raumtemperatur[a];
		for (int i = 0; i < a; i++) {
			this.lager_25[i] = new Lagerplatz_Raumtemperatur();
		}

		this.lager_5 = new Lagerplatz_gekuehlt[b];
		for (int i = 0; i < b; i++) {
			this.lager_5[i] = new Lagerplatz_gekuehlt();
		}

		this.lager_minus18 = new Lagerplatz_tiefgekuehlt_minus18[c];
		for (int i = 0; i < c; i++) {
			this.lager_minus18[i] = new Lagerplatz_tiefgekuehlt_minus18();
		}

		this.lager_minus30 = new Lagerplatz_tiefgekuehlt_minus30[d];
		for (int i = 0; i < d; i++) {
			this.lager_minus30[i] = new Lagerplatz_tiefgekuehlt_minus30();
		}
	}

	/**
	 * Diese Methode legt eine Ware in das Lager, sofern Platz vorhanden ist.
	 * 
	 * @param w
	 *            Die zu lagernde Ware.
	 * @return true wenn die Ware erfolgreich gelagert wurde, ansonsten false.
	 */
	public boolean store(Ware w) {
		boolean firstAttempt = false;
		firstAttempt = store(lager_25, w) || store(lager_5, w) || store(lager_minus18, w) || store(lager_minus30, w);
		if (firstAttempt == true) {
			return true;
		}

		boolean secondAttempt = false;
		secondAttempt = store2(lager_25, w) || store2(lager_5, w) || store2(lager_minus18, w)
				|| store2(lager_minus30, w);
		if (secondAttempt == true) {
			return true;
		}

		return false;
	}

	/**
	 * Diese Metohode legt eine Ware in eine bestimmte Art von Lagerpltzen, sofern Platz vorhanden ist.
	 * 
	 * @param l
	 *            Art von Lagerpltzen, in die die Ware gelegt werden soll.
	 * @param w
	 *            Die zu lagernde Ware.
	 * @return true wenn die Ware erfolgreich gelagert wurde, ansonsten false.
	 */
	private boolean store(Lagerplatz[] l, Ware w) {
		for (int i = 0; i < l.length; i++) {
			if (l[i].getWare() == null) {
				return l[i].store(w, false);
			}
		}

		return false;
	}

	/**
	 * Diese Metohode startet den zweiten Versuch, eine Ware in eine bestimmte Art von Lagerpltzen zweiter Wahl zu
	 * legen, wenn der erste Versuch aufgrund von Platzmangel fehlschlug. Vorbedingung ist, dass vor Aufruf dieser
	 * Methode bereits ein erster Versuch, die Ware zu lagern, stattgefunden hat!
	 * 
	 * @param l
	 *            Art von Lagerpltzen zweiter Wahl, in die die Ware gelegt werden soll.
	 * @param w
	 *            Die zu lagernde Ware.
	 * @return true wenn die Ware erfolgreich gelagert wurde, ansonsten false.
	 */
	private boolean store2(Lagerplatz[] l, Ware w) {
		for (int i = 0; i < l.length; i++) {
			if (l[i].getWare() == null) {
				return l[i].store(w, true);
			}
		}

		return false;
	}

	/**
	 * Diese Methode entfernt eine Bestimmte Ware vom Lager.
	 * 
	 * @param snr
	 *            Seriennummer der zu entfernenden Ware.
	 * @return Referenz auf die entfernte Ware. Falls keine Ware entfernt wurde, wird null zurckgeliefert.
	 */
	public Ware remove(int snr) {
		Ware temp;

		temp = lager_remove(lager_25, snr);
		if (temp != null) {
			return temp;
		}

		temp = lager_remove(lager_5, snr);
		if (temp != null) {
			return temp;
		}

		temp = lager_remove(lager_minus18, snr);
		if (temp != null) {
			return temp;
		}

		temp = lager_remove(lager_minus30, snr);
		if (temp != null) {
			return temp;
		}

		return null;
	}

	/**
	 * Diese Methode entfernt eine Bestimmte Ware von bestimmten Lagerpltzen vom Lager.
	 * 
	 * @param l
	 *            Art von Lagerpltzen, von der die Ware entfernt werden soll.
	 * @param snr
	 *            Seriennummer der zu entfernenden Ware.
	 * @return Referenz auf die entfernte Ware. Falls keine Ware entfernt wurde, wird null zurckgeliefert.
	 */
	private Ware lager_remove(Lagerplatz[] l, int snr) {
		Ware temp;

		for (int i = 0; i < l.length; i++) {
			if (l[i].getWare() != null && l[i].getWare().getSNR() == snr) {
				temp = l[i].getWare();
				l[i].reset();
				return temp;
			}
		}

		return null;
	}

	/**
	 * Diese Methode gibt die momentane Auslastung des Lagers in der Standardausgabe aus.
	 */
	public void utilization() {
		int lager_25_count = lager_utilization(lager_25);
		int lager_5_count = lager_utilization(lager_5);
		int lager_minus_18_count = lager_utilization(lager_minus18);
		int lager_minus_30_count = lager_utilization(lager_minus30);

		System.out.println("gesamt Auslastung: \t\t"
				+ (lager_25_count + lager_5_count + lager_minus_18_count + lager_minus_30_count) + "/"
				+ (lager_25.length + lager_5.length + lager_minus18.length + lager_minus30.length));
		System.out.println("Raumtemperatur Auslastung: \t" + lager_25_count + "/" + lager_25.length);
		System.out.println("gekhlt Auslastung: \t\t" + lager_5_count + "/" + lager_5.length);
		System.out.println("tiefgekhlt -18C Auslastung: \t" + lager_minus_18_count + "/" + lager_minus18.length);
		System.out.println("tiefgekhlt -30C Auslastung: \t" + lager_minus_30_count + "/" + lager_minus30.length);
	}

	/**
	 * Diese Methode kalkuliert die momentane Auslastung bestimmter Lagerpltze des Lagers.
	 * 
	 * @param l
	 *            Art von Lagerpltzen, dessen Auslastung kalkuliert werden soll.
	 * @return Anzahl der belegten Lagerpltze.
	 */
	private int lager_utilization(Lagerplatz[] l) {
		int count = 0;
		for (int i = 0; i < l.length; i++) {
			if (l[i].getWare() != null) {
				count++;
			}
		}

		return count;
	}

	/**
	 * Diese Methode gibt die momentan im Lager befindlichen Waren in der Standardausgabe aus.
	 */
	public void inventar() {
		lager_inventar(lager_25);
		lager_inventar(lager_5);
		lager_inventar(lager_minus18);
		lager_inventar(lager_minus30);
	}

	/**
	 * Diese Methode gibt die momentan im Lager in bestimmten Lagerpltzen befindlichen Waren in der Standardausgabe
	 * aus.
	 * 
	 * @param l
	 *            Art von Lagerpltzen, dessen Waren-Inhalt ausgegeben werden soll.
	 */
	private void lager_inventar(Lagerplatz[] l) {
		for (int i = 0; i < l.length; i++) {
			if (l[i].getWare() != null) {
				System.out.println("Seriennummer: " + l[i].getWare().getSNR() + ", Name: " + l[i].getWare().getName());
			}
		}
	}
}
