/**
 * Diese Klasse stellt Lagerpltze zur Verfgung. Ein Lagerplatz kann eine Ware lagern.
 * 
 * @author SK
 * 
 */
public abstract class Lagerplatz {
	protected Ware w;

	/**
	 * Default-Konstruktor, liefert ein leeres Objekt vom Typ Lagerplatz.
	 */
	public Lagerplatz() {
		this.w = null;
	}

	/**
	 * Dieser Konstruktor erzeugt einen neuen Lagerplatz, der eine Ware lagert.
	 * 
	 * @param w
	 *            Die zu lagernde Ware.
	 */
	public Lagerplatz(Ware w) {
		this.w = w;
	}

	/**
	 * Diese Methode gibt die im Lagerplatz befindliche Ware aus.
	 * 
	 * @return die im Lagerplatz befindliche Ware.
	 */
	public Ware getWare() {
		return w;
	}

	/**
	 * Diese Methode lagert eine (neue) Ware in den Lagerplatz. WICHTIG!!! Vorbedingung ist, dass unbedingt berprft
	 * werden muss, ob die Ware kompatibel zu dem Lagerplatz ist!
	 * 
	 * @param w
	 *            Die zu lagernde Ware.
	 */
	protected void store(Ware w) {
		this.w = w;
	}

	/**
	 * Diese Methode setzt die Ware zurck.
	 */
	public void reset() {
		this.w = null;
	}

	/**
	 * Diese Methode versucht, eine Ware im Lagerplatz abzulagern. Als Vorbedingung muss unbedingt angegeben werden, ob
	 * es sich hierbei um den ersten oder zweiten Versuch handelt!
	 * 
	 * @param w
	 *            Die zu lagernde Ware.
	 * @param second_attempt
	 *            Angabe, ob es sich hierbei um den ersten oder zweiten Versuch handelt.
	 * @return true wenn die Ware erfolgreich gelagert wurde, ansonsten false.
	 */
	public abstract boolean store(Ware w, boolean second_attempt);
}
