/**
 * Diese Klasse stellt Waren zur Verfgung. Eine Ware kann in einem Lagerplatz gelagert werden.
 * 
 * @author SK
 * 
 */
public abstract class Ware {
	protected int Seriennummer;
	protected String Name;

	/**
	 * Default-Konstruktor, erzeugt ein leeres Objekt vom Typ Ware.
	 */
	public Ware() {
		this.Seriennummer = -1;
		this.Name = "";
	}

	/**
	 * Dieser Konstruktor erzeugt eine neue Ware mit bestimmter Seriennummer und bestimmtem Namen.
	 * 
	 * @param snr
	 *            Seriennummer der neuen Ware.
	 * @param name
	 *            Name der neuen Ware.
	 */
	public Ware(int snr, String name) {
		this.Seriennummer = snr;
		this.Name = name;
	}

	/**
	 * Diese Methode liefert die Seriennummer der Ware.
	 * 
	 * @return Seriennummer der Ware.
	 */
	public int getSNR() {
		return this.Seriennummer;
	}

	/**
	 * Diese Methode liefert den Namen der Ware.
	 * 
	 * @return Name der Ware.
	 */
	public String getName() {
		return this.Name;
	}

	/**
	 * Diese Methode fgt die Ware einem Lagerplatz bei Raumtemperatur hinzu, falls mglich. Auerdem wird angegeben, ob
	 * es sich hierbei um den ersten oder zweiten Versuch handelt.
	 * 
	 * @param l
	 *            Lagerplatz bei Raumtemperatur, in der die Ware gelagert werden soll.
	 * @return true wenn die Ware erfolgreich gelagert wurde, ansonsten false.
	 */
	public abstract boolean storedInRaumtemperatur(Lagerplatz_Raumtemperatur l, boolean second_attempt);

	/**
	 * Diese Methode fgt die Ware einem gekhlten Lagerplatz hinzu, falls mglich. Auerdem wird angegeben, ob es sich
	 * hierbei um den ersten oder zweiten Versuch handelt.
	 * 
	 * @param l
	 *            Gekhlter Lagerplatz, in der die Ware gelagert werden soll.
	 * @return true wenn die Ware erfolgreich gelagert wurde, ansonsten false.
	 */
	public abstract boolean storedInGekuehlt(Lagerplatz_gekuehlt l, boolean second_attempt);

	/**
	 * Diese Methode fgt die Ware einem auf -18C tiefgekhlten Lagerplatz hinzu, falls mglich. Auerdem wird
	 * angegeben, ob es sich hierbei um den ersten oder zweiten Versuch handelt.
	 * 
	 * @param l
	 *            Auf -18C gekhlter Lagerplatz, in der die Ware gelagert werden soll.
	 * @return true wenn die Ware erfolgreich gelagert wurde, ansonsten false.
	 */
	public abstract boolean storedInTiefgekuehlt18(Lagerplatz_tiefgekuehlt_minus18 l, boolean second_attempt);

	/**
	 * Diese Methode fgt die Ware einem auf -30C tiefgekhlten Lagerplatz hinzu, falls mglich. Auerdem wird
	 * angegeben, ob es sich hierbei um den ersten oder zweiten Versuch handelt.
	 * 
	 * @param l
	 *            Auf -30C gekhlter Lagerplatz, in der die Ware gelagert werden soll.
	 * @return true wenn die Ware erfolgreich gelagert wurde, ansonsten false.
	 */
	public abstract boolean storedInTiefgekuehlt30(Lagerplatz_tiefgekuehlt_minus30 l, boolean second_attempt);
}
